#include "post.h"
#include "cgi.h"
#include "ieee.h"
#include <string.h>

/* This program provides a CLI interface to the IEEE database functions
 * 
 *    It is probably best used for testing and debugging the functionality
 *    of the web interface.
 */

 void print_help(){
	printf("Command Summary:\n");
	printf(" help - prints help\n");
	printf(" insert key_pairs\n");
	printf("    type of insert figured out based on key pairs given\n");
	printf(" update key_pairs\n");
	printf("    updates table person with values from key pairs\n");
	printf(" view order order=number\n");
	printf(" total inv type=x sub_type=y color=z size=w\n");
	printf(" view expense \n");
	printf(" echo stuff\n");
	printf("   copies stuff to stdout\n");
	printf(" gen list key_pairs\n");
	printf("   generates a html list based on key pairs\n");
	printf(" quit - quits\n");
	printf("Note:    Key_pairs is in the form of field1=val1 field2=val2 ...\n");
	printf("Example: insert inventory=1 type=tshirt color=red price=12.22\n");
 }


 /* Append and parse are intended to be used together. They parse
  *  key=value pairs and construct a QUERY_STRING from it
  */

 void append(char* append_string) {
	char buf[1024];
	if ( getenv ("QUERY_STRING") ) {
		strncpy(buf, getenv("QUERY_STRING"), 1023);
		strncat(buf, "&", 1024-strlen(buf));
		strncat(buf, append_string, 1024-strlen(buf));
		buf[1023]=0;
		setenv ("QUERY_STRING", buf, 1);
	} else
		setenv ("QUERY_STRING", append_string, 1);

 }

void parse(char* buf) {
	char *str1, *str2;
	str1=str2=buf;
	while (str2) {
		str1=strsep(&str2, " ");
		if (*str1) 
			append(str1);
	}

	printf("Query String = '%s'\n",getenv("QUERY_STRING"));


}

/*
 * Strips the dot from a string in the form of $'xxxx.xx'
 *  gets string from keypairs using cgi library where
 *  key=variable_name
 */
int strip_dot(char* variable_name) {
	char  *str1, *str2;
	str2=get_data(variable_name);
	if (str2) { /* strips dot from $xxxxx.xx */
		str1=strsep(&str2, ".");
		strncat(str1, str2, 10);
		return atoi(str1);
	}
	return 0;
}

/*
 * The fill_* functions fill their respective structs with data from
 *            the QUERY_STRING environment variable (or POST).
 *
 *  fill_receipt - also associates a order_form with a list of receipts
 */
inventory_type* fill_inventory() {
	static inventory_type inventory;
	inventory.type=get_data("type");
	inventory.sub_type=get_data("sub_type");
	inventory.size=get_data("size");
	inventory.color=get_data("color");
	inventory.description=get_data("description");

	inventory.price=strip_dot("price");

	if (get_data("stock") )
		inventory.stock=atoi(get_data("stock"));
	else
		inventory.stock=0;

	return &inventory;
}

void fill_expense() {
	expense_type expense;

	if (!get_data("person")) {
		printf("Error: Expense insertion requires a person to be\n");
		printf("       associated with the expense.\n\n");
		return;
	}
	if (!get_data("amount")) {
		printf("Error: Expense insertion requires amount be specified\n");
		return;
	}

	expense.account=get_data("account");
	expense.amount=strip_dot("amount");
	expense.description=get_data("description");
	insert_expense(&expense, atoi(get_data("who")));
}

void fill_receipt() {
	receipt_type receipt;
	int order;

	if (!get_data("order")){
		/* create order form */
		order=1; /* <-- temporary solution */
	} else
		order=atoi(get_data("order"));

	if (!get_data("inventory")) {
		printf("Error: Receipt insertion requires a associated inventory\n");
		return ;
	} 

	if (get_data("tax")) 
		receipt.tax=atoi(get_data("tax"));
	else
		receipt.tax=0;

	receipt.price=strip_dot("price");
	receipt.description=get_data("description");
	receipt.account=get_data("account");
	insert_receipt(&receipt, order, atoi(get_data("inventory")));
}

persocom* fill_person() {
	static persocom person;
	if (get_data("ieee_number"))
		person.ieee_number=atoi(get_data("ieee_number"));
	else
		person.ieee_number=0;

	if (get_data("zip"))
		person.zip=atoi(get_data("zip"));
	else
		person.zip=0;

	if (get_data("phone"))
		person.phone=atoi(get_data("phone"));
	else
		person.phone=0;

	if (get_data("serial"))
		person.serial=atoi(get_data("serial"));
	else
		person.serial=0;

	person.url=get_data("url");
	person.real_name=get_data("real_name");
	person.user_name=get_data("user_name");
	person.password=get_data("password");
	person.email=get_data("email");
	person.street=get_data("street");
	person.city=get_data("city");
	person.description=get_data("description");

	return &person;

}

/* insert() checks QUERY_STRING for certain variables
 *    if they exist it calls other functions which 
 *    will attempt to add the anticapated data into
 *    their database.
 */   
void insert() {
	if (get_data("inventory")) 
		insert_inventory(fill_inventory());
	else if (get_data("expense")) 
		fill_expense();
	else if (get_data("receipt")) 
		fill_receipt();
	else if (get_data("person"))
		insert_person(fill_person());
	else
		printf("Please specify a table to insert into\n");
}

/*
 * From the CS180 example code
 */
void pretty_print(PGresult *res) {
	PQprintOpt print_opts;

	memset( &print_opts, '\0', sizeof( print_opts ));
	print_opts.header = 1;
	print_opts.align = 1;
	print_opts.html3 = 0;
	print_opts.fieldSep = "|";
	PQprint( stdout, res, &print_opts );

}

void view_order() {
	char buf[1024];

	if (!get_data("order")) {
		printf("Error: Missing serial for orderform query");
		return;
	}

	printf("\n====---- Items\n\n");

	sprintf(buf, "select inventory.type, inventory.sub_type, inventory.size, inventory.color, inventory.def_price from inventory, receipt where receipt.order_num=%s and receipt.item_num=inventory.item_num",
		get_data("order"));
	pretty_print(exec(buf));

	printf("====---- Total\n\n");

	sprintf(buf, "select sum(inventory.def_price) from inventory, receipt where receipt.order_num=%s and receipt.item_num=inventory.item_num",
		get_data("order"));
	pretty_print(exec(buf));

	printf("====---- Paid\n\n");
	sprintf(buf, "select paid from order_form where order_num=%s",
		get_data("order"));
	pretty_print(exec(buf));


}

void total_inventory(char *type, char *sub_type, char* size, char* color) {
	char buf[1024];
	sprintf(buf, "select count (*) as total from receipt where item_num IN (select item_num from inventory where type='%s' and sub_type='%s' and size='%s' and color='%s') AND (select count (*) from order_form where paid=1 AND order_num=receipt.order_num) > 0",
		type, sub_type, size, color);
	pretty_print(exec(buf));
}

void total_inv() {
	total_inventory(get_data("type"), get_data("sub_type"), get_data("size"),
					get_data("color"));
}

void view_expense() {
	char buf[1024];
	sprintf(buf, "select amount, account, who, ds from expense");
	pretty_print(exec(buf));
}

/* generate_list_inventory generate a select statement where 
 * list variable indicates the values to be returned, and 
 * the constraints (where clause) are the variables type,
 * sub_type, color, and size as well as having a total > 0
 */
char* generate_list_inventory(inventory_type *inventory) {
	static char buf[1024];
	char append[256];
	if (!get_data("list_value")) {
		printf("Error list_value environment variable is not present\n");
		return 0;
	}
	sprintf(buf, "select distinct %s from inventory where ", get_data("list_value"));

	if (get_data("type")) {
		sprintf(append,"type=%s ", get_data("type"));
		strcat(buf,append);
	}
	if (get_data("sub_type")) {
		sprintf(append,"sub_type=%s ", get_data("sub_type"));
		strcat(buf,append);
	}
	if (get_data("color")) {
		sprintf(append,"color=%s ", get_data("color"));
		strcat(buf,append);
	}
	if (get_data("size")) {
		sprintf(append,"size=%s ", get_data("size"));
		strcat(buf,append);
	}

	/* where total > xxx */
	return buf;

}

void option_list() {
	PGresult *res;
	
	while ( PQntuples(res=exec("fetch forward from csor")) ) {
		printf("\t<OPTION VALUE=\"%s\">%s\n",PQgetvalue(res,0,0),PQgetvalue(res,0,0);
	}
	exec("close csor");
}

/* executes query string and stores the result into a cursor.
 *  once the result is stored, it looks at the value of list_query
 *  to determine how to display the data, and then calls the appropriate
 *  function to iterate through the cursor. after the function has
 *  completed, it finishes the transaction.
 */
void cursor_itor(char *query_string) {
	char buf[1024];
	if (!query_string || strlen(query_string) < 10 || strlen(query_string) > 950) {
		if (!query_string)
			printf("Query string is null\n\n");
		else
			printf("Quersy string '%s' fails bounds checking\n\n", query_string);
		return ;
	}
	exec("begin transaction");
	sprintf(buf,"declare csor cursor for %s", query_string);
	exec(buf);
	if (get_data("list_query")) {
		if (!strncasecmp(get_data("list_query"), "option", 6)) 
			option_list();
		else {
			printf("Unknown list type '%s',\n",get_data("list_query"));
			printf("known types = option\n\n");
		}


	} else
		printf("list_query environment variable not present\n\n");
	exec("end transaction");
}


/*
 * generate list from table, currently only inventory type is supported.
 *
 * ex. gen list inventory=1 list_value=color list_query=option type=1
 */
void generate_list() {
	if (get_data("inventory")) 
		cursor_itor(generate_list_inventory(fill_inventory()));
	else
		printf("unkown table type\n");

}


int main() {
	char cmd[80], *cmd_ptr;
	setenv("NO_HEADERS", "TRUE", 1);
	printf("IEEE Databears Interface\n");
	printf("   type 'help' for help\n");

	while(1) {
		printf("> ");
		fgets(cmd, 75, stdin);
		setenv ("RESTART", "YES", 1);
		unsetenv ("QUERY_STRING");

		if (!cmd) 
			break;
		if (cmd[strlen(cmd)-1]== '\n'); /* strip new line */
		cmd[strlen(cmd)-1]=0; 

		cmd_ptr=cmd;
		while (*cmd_ptr && (*cmd_ptr==' '))
			cmd_ptr++;


		if ( strncasecmp(cmd_ptr, "help", 4) == 0 )
			print_help(); 

		else if ( strncasecmp(cmd, "insert", 6) == 0 ) {
			cmd_ptr=cmd_ptr+6;
			(char*)cmd_ptr=strsep(&cmd_ptr, "\n");
			parse(cmd_ptr);
			insert();
		} 
		else if ( strncasecmp(cmd_ptr, "update", 6) == 0 ) {
			cmd_ptr=cmd_ptr+6;
			(char*)cmd_ptr=strsep(&cmd_ptr, "\n");
			parse(cmd_ptr);
			update_person(fill_person());
		} 
		else if (strncasecmp(cmd_ptr, "view order", 10) == 0 ) {
			cmd_ptr=cmd_ptr+10;
			(char*)cmd_ptr=strsep(&cmd_ptr, "\n");
			parse(cmd_ptr);
			view_order();
		}
		else if (strncasecmp(cmd_ptr, "total inv ", 10) == 0 ) {
			cmd_ptr=cmd_ptr+10;
			(char*)cmd_ptr=strsep(&cmd_ptr, "\n");
			parse(cmd_ptr);
			total_inv();
		}
		else if (strncasecmp(cmd_ptr, "view expense", 12) == 0 ) {
			cmd_ptr=cmd_ptr+12;
			(char*)cmd_ptr=strsep(&cmd_ptr, "\n");
			parse(cmd_ptr);
			view_expense();
		}
		else if (strncasecmp(cmd_ptr, "echo", 4) == 0 ) {
			cmd_ptr=cmd_ptr+4;
			printf("%s",cmd_ptr);
		}
		else if (strncasecmp(cmd_ptr, "gen list", 8) == 0 ) {
			cmd_ptr=cmd_ptr+8;
			parse(cmd_ptr);
			generate_list();
		}

		else if ( strncasecmp(cmd_ptr, "quit", 4) == 0)
			return(1); else
		printf("Invalid Command\n\n");




		fpurge(stdin);
	}

	return(0);
}
